-------------------------------------------------------------------------------
-- ElvUI Chat Tweaks By Lockslap (US, Bleeding Hollow)
-- <Borderline Amazing>, http://ba-guild.com
-- Based on functionality provided by Prat and/or Chatter
-------------------------------------------------------------------------------
local Module	= ElvUI_ChatTweaks:NewModule("Auto Scroll")
local asFrame	= CreateFrame("Frame")
local L			= LibStub("AceLocale-3.0"):GetLocale("ElvUI_ChatTweaks")
Module.name		= L["Auto Scroll"]

local format = string.format

local handlers, running, scrollDowns = {}, {}, {}
local scrollFuncs = {"ScrollUp", "ScrollDown", "ScrollToTop", "PageUp", "PageDown"}

local db
local options = {
	interval = {
		type	= "range",
		name	= L["Interval"],
		desc	= L["Time, in seconds, to allow to pass before automatically scrolling to the bottom of a chat window."],
		get		= function() return db.interval end,
		set		= function(_, value) db.interval = value; Module:SetupAutoScroll() end,
	}
}
local defaults = {
	profile = {
		interval = 30
	}
}

local function register(name, func, rate, ...)
	handlers[name] = {
		name	= name,
		func	= func,
		rate	= rate or 0,
		...
	}
end

local function start(name)
	handlers[name].elapsed = 0
	running[name] = true
	asFrame:Show()
end

local function stop(name)
	running[name] = nil
	if not next(running) then asFrame:Hide() end
end

local function resetFrame(name, frame)
	stop(name .. "DownTimeout")
	start(name .. "DownTick")
	
	if frame.downButton:IsShown() then
		frame.downButton:Hide()
		frame.downButton:UnlockHighlight()
	end
end

local function scrollOnce(name, frame)
	if frame:AtBottom() then
		stop(name .. "DownTick")
	else
		scrollDowns[name](frame)
	end
end

local function OnUpdate(frame, elapsed)
	for name, value in pairs(handlers) do
		if running[name] then
			value.elapsed = value.elapsed + elapsed
			if value.elapsed >= value.rate then
				value.func(unpack(value))
				value.elapsed = 0
			end
		end
	end
end

function Module:SetupAutoScroll()
	for i = 1, NUM_CHAT_WINDOWS do
		local name = format("ChatFrame%d", i)
		local frame = _G[name]
		scrollDowns[name] = frame.ScrollDown
		register(name .. "DownTick", scrollOnce, 0.1, name, frame)
		register(name .. "DownTimeout", resetFrame, db.interval or 30, name, frame)
		for _, func in ipairs(scrollFuncs) do
			local orig = frame[func]
			frame[func] = function(...)
				stop(name .. "DownTick")
				start(name .. "DownTimeout", 1)
				orig(...)
			end
		end
	end
end

function Module:OnEnable()
	self:SetupAutoScroll()
	asFrame:SetScript("OnUpdate", OnUpdate)
end

function Module:OnDisable()
	asFrame:SetScript("OnUpdate", nil)
end

function Module:OnInitialize()
	self.db = ElvUI_ChatTweaks.db:RegisterNamespace("AutoScroll", defaults)
	db = self.db.profile
end

function Module:GetOptions()
	return options
end

function Module:Info()
	return L["Automatically scroll to the bottom of the chat window after |cff00ff00X|r seconds."]
end