-------------------------------------------------------------------------------
-- ElvUI Chat Tweaks By Lockslap (US, Bleeding Hollow)
-- <Borderline Amazing>, http://ba-guild.com
-- Based on functionality provided by Prat and/or Chatter
-------------------------------------------------------------------------------
local Module		= ElvUI_ChatTweaks:NewModule("Hyperlink Hover", "AceHook-3.0")
local L				= LibStub("AceLocale-3.0"):GetLocale("ElvUI_ChatTweaks")
Module.name			= L["Hyperlink Hover"]

-- module vars
local match = string.match
local hyperlinkTypes = {}
local db

-------------------------------------------------------------------------------
--[[ Module Options/Defaults ]]--
local options = {
	combat			= {
		type	= "toggle",
		order	= 14,
		name	= L["Disable in Combat"],
		desc	= L["Disable the hovering while in combat."],
		width	= "full",
		get		= function() return db.combat end,
		set		= function(_, value) db.combat = value end,
	},
	item			= {
		type	= "toggle",
		name	= L["Items"],
		desc	= L["Item hyperlinks."],
		get		= function() return db.item end,
		set		= function(_, value) db.item = value; buildHyperlinks() end,
	},
	spell			= {
		type	= "toggle",
		name	= L["Spells"],
		desc	= L["Spell hyperlinks."],
		get		= function() return db.spell end,
		set		= function(_, value) db.spell = value; buildHyperlinks() end,
	},
	unit			= {
		type	= "toggle",
		name	= L["Units"],
		desc	= L["Unit hyperlinks."],
		get		= function() return db.unit end,
		set		= function(_, value) db.unit = value; buildHyperlinks() end,
	},
	quest			= {
		type	= "toggle",
		name	= L["Quests"],
		desc	= L["Quest hyperlinks."],
		get		= function() return db.quest end,
		set		= function(_, value) db.quest = value; buildHyperlinks() end,
	},
	enchant			= {
		type	= "toggle",
		name	= L["Enchants"],
		desc	= L["Enchant hyperlinks."],
		get		= function() return db.enchant end,
		set		= function(_, value) db.enchant = value; buildHyperlinks() end,
	},
	achievement		= {
		type	= "toggle",
		name	= L["Achievements"],
		desc	= L["Achievement hyperlinks."],
		get		= function() return db.achievement end,
		set		= function(_, value) db.achievement = value; buildHyperlinks() end,
	},
	instancelock	= {
		type	= "toggle",
		name	= L["Instance Locks"],
		desc	= L["Instance lock hyperlinks."],
		get		= function() return db.instancelock end,
		set		= function(_, value) db.instancelock = value; buildHyperlinks() end,
	},
	talent			= {
		type	= "toggle",
		name	= L["Talents"],
		desc	= L["Talent hyperlinks."],
		get		= function() return db.talent end,
		set		= function(_, value) db.talent = value; buildHyperlinks() end,
	},
	glyph			= {
		type	= "toggle",
		name	= L["Glyphs"],
		desc	= L["Glyph hyperlinks."],
		get		= function() return db.glyph end,
		set		= function(_, value) db.glyph = value; buildHyperlinks() end,
	}
}

local defaults = {
	profile	= {
		combat			= true,
		item			= true,
		spell			= true,
		unit			= true,
		quest			= true,
		enchant			= true,
		achievement		= true,
		instancelock	= true,
		talent			= true,
		glyph			= true
	}
}
-------------------------------------------------------------------------------

-------------------------------------------------------------------------------
--[[ Module Functions ]] --
local function buildHyperlinks()
	hyperlinkTypes = {
		item			= db.item,
		spell			= db.spell,
		unit			= db.unit,
		quest			= db.quest,
		enchant			= db.enchant,
		achievement		= db.achievement,
		instancelock	= db.instancelock,
		talent			= db.talent,
		glyph			= db.glyph
	}
end

function Module:OnHyperlinkEnter(self, refString)
	local linkToken = refString:match("^([^:]+)")
	if db.combat and InCombatLockdown() then return end
	buildHyperlinks()
	if hyperlinkTypes[linkToken] then
		ShowUIPanel(GameTooltip)
		GameTooltip:SetOwner(self, "ANCHOR_CURSOR")
		GameTooltip:SetHyperlink(refString)
		GameTooltip:Show()
	end
end

function Module:OnHyperlinkLeave(self, refString)
	local linkToken = refString:match("^([^:]+)")
	buildHyperlinks()
	if hyperlinkTypes[linkToken] then
		HideUIPanel(GameTooltip)
	end
end
-------------------------------------------------------------------------------

-------------------------------------------------------------------------------
--[[ Ace3 Framework Events ]]--
function Module:OnInitialize()
	self.db = ElvUI_ChatTweaks.db:RegisterNamespace("HyperlinkHover", defaults)
	db = self.db.profile
end

function Module:OnEnable()
	for i = 1, NUM_CHAT_WINDOWS do
		local frame = _G["ChatFrame" .. i]
		self:HookScript(frame, "OnHyperlinkEnter")
		self:HookScript(frame, "OnHyperlinkLeave")
	end
	buildHyperlinks()
end

function Module:Info()
	return L["Hovering over a hyperlink in chat will display it's tooltip."]
end

function Module:GetOptions()
	return options
end
-------------------------------------------------------------------------------