-------------------------------------------------------------------------------
-- ElvUI Chat Tweaks By Lockslap (US, Bleeding Hollow)
-- <Borderline Amazing>, http://ba-guild.com
-- Based on functionality provided by Prat and/or Chatter
-------------------------------------------------------------------------------
local Module	= ElvUI_ChatTweaks:NewModule("Invite Links", "AceEvent-3.0", "AceHook-3.0")
local L			= LibStub("AceLocale-3.0"):GetLocale("ElvUI_ChatTweaks")
Module.name		= L["Invite Links"]

local gsub		= string.gsub
local format	= string.format
local sub		= string.sub
local match		= string.match

local events	= {}
local style		= "|cffffffff|Hinvite:%s|h[%s]|h|r"
local chatEvent, chatEventTarget

local options = {
	addWord = {
		type	= "input",
		order	= 13,
		name	= L["Add Word"],
		desc	= L["Add word to your invite trigger list"],
		get		= function() end,
		set		= function(_, value) db.words[value:lower()] = value end
	},
	removeWord = {
		type 	= "select",
		order	= 14,
		name 	= L["Remove Word"],
		desc	= L["Remove a word from your invite trigger list"],
		get 	= function() end,
		set 	= function(_, value) db.words[value:lower()] = nil end,
		values 	= function() return db.words end,
		confirm	= function() return (L["Really remove this word from your trigger list?"]) end
	},
	altClick = {
		type 	= "toggle",
		name 	= L["Alt-click name to invite"],
		width 	= "full",
		desc 	= L["Lets you alt-click player names to invite them to your party."],
		get		= function() return db.altClickToInvite end,
		set 	= function(_, value) db.altClickToInvite = value end
	},
	validEvents = {
		type	= "group",
		inline	= true,
		name	= "Valid Events",
		args	= {
			description		= {
				type	= "description",
				order	= 1,
				width	= "full",
				name	= L["Here you can select which channels this module will scan for the keywords to trigger the invite."],
			},
			chatMsgSay		= {
				type	= "toggle",
				name	= L["Say"],
				desc	= L["Say Chat"],
				get		= function() return db.chatMsgSay end,
				set		= function(_, value) db.chatMsgSay = value; Module:BuildEvents() end,
			},
			chatMsgYell		= {
				type	= "toggle",
				name	= L["Yell"],
				desc	= L["Yell Chat"],
				get		= function() return db.chatMsgYell end,
				set		= function(_, value) db.chatMsgYell = value; Module:BuildEvents() end,			
			},
			chatMsgWhisper	= {
				type	= "toggle",
				name	= L["Whisper"],
				desc	= L["Whispers"],
				get		= function() return db.chatMsgWhisper end,
				set		= function(_, value) db.chatMsgWhisper = value; Module:BuildEvents() end,			
			},
			chatMsgOfficer	= {
				type	= "toggle",
				name	= L["Officer"],
				desc	= L["Officer Chat"],
				get		= function() return db.chatMsgOfficer end,
				set		= function(_, value) db.chatMsgOfficer = value; Module:BuildEvents() end,			
			},
			chatMsgGuild	= {
				type	= "toggle",
				name	= L["Guild"],
				desc	= L["Guild Chat"],
				get		= function() return db.chatMsgGuild end,
				set		= function(_, value) db.chatMsgGuild = value; Module:BuildEvents() end,			
			},
		}
	},
}

local defaults = {
	profile = {
		words				= {},
		altClickToInvite	= true,
		chatMsgSay			= true,
		chatMsgYell			= true,
		chatMsgWhisper		= true,
		chatMsgOfficer		= true,
		chatMsgGuild		= true,
	}
}

local function addLinks(m, t, p)
	if db.words[t:lower()] and p ~= "_" then
		t = format(style, chatEventTarget, t)
		return t .. p
	end
	return m
end

function Module:BuildEvents()
	events = {
		CHAT_MSG_SAY		= db.chatMsgSay and true or false,
		CHAT_MSG_YELL		= db.chatMsgYell and true or false,
		CHAT_MSG_WHISPER	= db.chatMsgWhisper and true or false,
		CHAT_MSG_GUILD		= db.chatMsgGuild and true or false,
		CHAT_MSG_OFFICER	= db.chatMsgOfficer and true or false
	}
end

function Module:AddMessage(frame, text, ...)
	if not text then return self.hooks[frame].AddMessage(frame, text, ...) end
	if events[chatEvent] and type(chatEventTarget) == "string" then
		text = gsub(text, "((%w+)(.?))", addLinks)
	end
	return self.hooks[frame].AddMessage(frame, text, ...)
end

function Module:ChatFrame_MessageEventHandler(frame, event, ...)
	chatEvent = event
	arg1, chatEventTarget = ...
	return self.hooks["ChatFrame_MessageEventHandler"](frame, event, ...)
end

function Module:SetItemRef(link, text, button)
	local linkType = sub(link, 1, 6)
	if IsAltKeyDown() and linkType == "player" and db.altClickToInvite then
		local name = match(link, "player:([^:]+)")
		InviteUnit(name)
		return nil	
	elseif linkType == "invite" then
		local name = sub(link, 8)
		InviteUnit(name)
		return nil
	end
	return self.hooks.SetItemRef(link, text, button)
end

function Module:OnEnable()
	if not next(db.words) then
		db.words[L["invite"]]	= L["invite"]
		db.words[L["inv"]]		= L["inv"]
	end
	
	for i = 1, NUM_CHAT_WINDOWS do
		local frame = _G[format("ChatFrame%d", i)]
		if frame ~= COMBATLOG then
			self:RawHook(frame, "AddMessage", true)
		end
	end
	self:RawHook(nil, "SetItemRef", true)
	self:RawHook("ChatFrame_MessageEventHandler", true)
	self:BuildEvents()
end

function Module:OnDisable()
	self:UnhookAll()
end

function Module:OnInitialize()
	self.db = ElvUI_ChatTweaks.db:RegisterNamespace("InviteLinks", defaults)
	db = self.db.profile
end

function Module:GetOptions()
	return options
end

function Module:Info()
	return L["Gives you more flexibility in how you invite people to your group."]
end