-------------------------------------------------------------------------------
-- ElvUI Chat Tweaks By Lockslap (US, Bleeding Hollow)
-- <Borderline Amazing>, http://ba-guild.com
-- Based on functionality provided by Prat and/or Chatter
-------------------------------------------------------------------------------
local Module	= ElvUI_ChatTweaks:NewModule("Player Names", "AceHook-3.0", "AceEvent-3.0")
local L			= LibStub("AceLocale-3.0"):GetLocale("ElvUI_ChatTweaks")
local AceTab	= LibStub("AceTab-3.0")
Module.name		= L["Player Names"]

local gsub		= string.gsub
local match		= string.match
local find		= string.find
local format	= string.format
local wipe		= table.wipe

local localNames	= {}
local localClass	= {}
local cache			= {}
local storedName	= {}
local player		= UnitName("player")

local channels = {
	GUILD	= {},
	PARTY	= {},
	RAID	= {}
}

local db
local options = {
	save = {
		type = "group",
		name = L["Save Data"],
		desc = L["Save data between sessions. Will increase memory usage"],
		args = {
			guild = {
				type = "toggle",
				name = L["Guild"],
				desc = L["Save class data from guild between sessions."],
				get = function()
					return db.saveGuild
				end,
				set = function(_,  v)
					db.saveGuild = v
					Module:UpdateSaveData(v)
				end
			},
			group = {
				type = "toggle",
				name = L["Group"],
				desc = L["Save class data from groups between sessions."],
				get = function()
					return db.saveGroup
				end,
				set = function(_,  v)
					db.saveGroup = v
					Module:UpdateSaveData(v)
				end
			},
			friend = {
				type = "toggle",
				name = L["Friends"],
				desc = L["Save class data from friends between sessions."],
				get = function()
					return db.saveFriends
				end,
				set = function(_,  v)
					db.saveFriends = v
					Module:UpdateSaveData(v)
				end
			},
			target = {
				type = "toggle",
				name = L["Target/Mouseover"],
				desc = L["Save class data from target/mouseover between sessions."],
				get = function()
					return db.saveTarget
				end,
				set = function(_,  v)
					db.saveTarget = v
					Module:UpdateSaveData(v)
				end
			},
			who = {
				type = "toggle",
				name = L["Who"],
				desc = L["Save class data from /who queries between sessions."],
				order = 104,
				get = function()
					return db.saveWho
				end,
				set = function(_,  v)
					db.saveWho = v
					Module:UpdateSaveData(v)
				end
			},
			saveAllWho = {
				type = "toggle",
				name = L["Save all /who data"],
				desc = L["Will save all data for large /who queries"],
				disabled = function() return not db.saveWho end,
				order = 105,
				get = function()
					return db.saveAllWho
				end,
				set = function(_,  v)
					db.saveAllWho = v
				end
			},
			resetDB = {
				type = "execute",
				name = L["Reset Data"],
				desc = L["Destroys all your saved class/level data"],
				width = "full",
				func = function() wipe(db.realm.names) end,
				order = 106,
				confirm = function() return L["Are you sure you want to delete all your saved class/level data?"] end
			}
		}
	},
	level = {
		type = "group",
		name = L["Player Level"],
		desc = L["Player level display options."],
		args = {
			includeLevel = {
				type = "toggle",
				name = L["Include level"],
				desc = L["Include the player's level"],
				order = 1,
				get = function() return db.includeLevel end,
				set = function(_,  val)
					db.includeLevel = val
					Module:WipeCache()
				end
			},
			separator = {
				type = "input",
				order = 2,
				name = L["Separator"],
				desc = L["Character to use between the name and level"],
				disabled = function() return not db.includeLevel end,
				get = function() return db.separator end,
				set = function(i, v)
					db.separator = v
				end
			},
			colorLevelByDifficulty = {
				type = "toggle",
				name = L["Color level by difficulty"],
				desc = L["Color level by difficulty"],
				order = 3,
				disabled = function() return not db.includeLevel end,
				get = function() return db.levelByDiff end,
				set = function(_,  v)
					db.levelByDiff = v
					Module:WipeCache()
				end,
			},
			excludeMaxLevel = {
				type = "toggle",
				name = L["Exclude max levels"],
				desc = L["Exclude level display for max level characters"],
				order = 4,
				disabled = function() return not db.includeLevel end,
				get = function() return db.excludeMaxLevel end,
				set = function(_,  val)
					db.excludeMaxLevel = val
					Module:WipeCache()
				end,
			},
		}
	},
	leftbracket = {
		type = "input",
		order = 13,
		name = L["Left Bracket"],
		desc = L["Character to use for the left bracket"],
		get = function() return db.leftBracket end,
		set = function(i, v)
			db.leftBracket = v
			leftBracket = v
		end
	},
	rightbracket = {
		type = "input",
		order = 14,
		name = L["Right Bracket"],
		desc = L["Character to use for the right bracket"],
		get = function() return db.rightBracket end,
		set = function(i, v)
			db.rightBracket = v
			rightBracket = v
		end
	},
	bnetHeader = {
		type = "header",
		name = L["Battle.Net Options"],
		order = 20
	},
	bnetBrackets = {
		type = "toggle",
		order = 21,
		name = L["RealID Brackets"],
		desc = L["Strip RealID brackets"],
		get = function() return db.bnetBrackets end,
		set = function(_, v)
			db.bnetBrackets = v
		end,
	},
	bnetRealNames = {
		type = "toggle",
		order = 22,
		name = L["No RealNames"],
		desc = L["Show toon names instead of real names"],
		get = function() return db.noRealNames end,
		set = function(_, v)
			db.noRealNames = v
		end,
	},
	useTabComplete = {
		type = "toggle",
		order = 19,
		name = L["Use Tab Complete"],
		desc = L["Use tab key to automatically complete character names."],
		get = function() return db.useTabComplete end,
		set = function(_,  v)
			db.useTabComplete = v
			if v and not AceTab:IsTabCompletionRegistered("ElvUI_ChatTweaks") then
				AceTab:RegisterTabCompletion("ElvUI_ChatTweaks", nil, tabComplete)
			elseif not v and AceTab:IsTabCompletionRegistered("ElvUI_ChatTweaks") then
				AceTab:UnregisterTabCompletion("ElvUI_ChatTweaks")
			end
		end
	},
	colorSelfInText = {
		type = "toggle",
		order = 17,
		name = L["Color self in messages"],
		desc = L["Color own charname in messages."],
		get = function() return db.colorSelfInText end,
		set = function(i, v)
			db.colorSelfInText = v
			colorSelfInText = v
		end
	},
	emphasizeSelfInText = {
		type = "toggle",
		order = 18,
		name = L["Emphasize Self"],
		desc = L["Add surrounding brackets to own charname in messages."],
		get = function() return db.emphasizeSelfInText end,
		set = function(i, v)
			db.emphasizeSelfInText = v
			emphasizeSelfInText = v
		end
	},
	colorBy = {
		type = "select",
		order = 15,
		name = L["Color Player Names By..."],
		desc = L["Select a method for coloring player names"],
		values = {
			CLASS	= L["Class"],
			NAME	= L["Name"],
			NONE	= L["None"],
		},
		get = function() return db.nameColoring end,
		set = function(_,  val)
			db.nameColoring = val
			Module:WipeCache()
		end
	},
	nickColor = {
		type = "color",
		order = 16,
		name = L["Default Name Color"],
		desc = L["The default color to use to color names."],
		get = function() return db.nickColor.r, db.nickColor.g, db.nickColor.b end,
		set = function(_, r, g, b)
			db.nickColor.r = r
			db.nickColor.g = g
			db.nickColor.b = b
		end
	},
}

local defaults = {
	realm	= {
		names	= {},
		levels	= {},
	},
	profile	= {
		saveData			= false,
		nameColoring		= "CLASS",
		leftBracket			= "[",
		rightBracket		= "]",
		levelByDiff			= true,
		bnetBrackets		= true,
		separator			= ":",
		useTabComplete		= true,
		colorSelfInText		= true,
		nickColor			= {r = 0.627, g = 0.627, b = 0.627},
		emphasizeSelfInText	= true,
		noRealNames			= false,	
	},
	global = {
		classes = {	
			DRUID		= L["Druid"],
			MAGE		= L["Mage"],
			PALADIN		= L["Paladin"],
			PRIEST		= L["Priest"],
			ROGUE		= L["Rogue"],
			HUNTER		= L["Hunter"],
			SHAMAN		= L["Shaman"],
			WARLOCK		= L["Warlock"],
			WARRIOR		= L["Warrior"],
			DEATHKNIGHT	= L["Death Knight"],
		}
	}
}

local origUnitClass = UnitClass
local function UnitClass(unit)
	local loc, sys = origUnitClass(unit)
	if sys and db.global.classes[sys] then
		db.global.classes[sys] = loc
		localClass = sys
	end
	return loc, sys
end

local tabComplete
do
	function tabComplete(t, text, pos)
		local word = text:sub(pos)
		if #word == 0 then return end
		local CF = ChatEdit_GetActiveWindow()
		local channel = CF:GetAttribute("chatType")
		if channel == "CHANNEL" then
			channel = select(2, GetChannelName(CF:GetAttribute("channelTarget"))):lower()
		elseif channel == "OFFICER" then
			channel = "GUILD"
		elseif channel == "RAID_WARNING" or channel == "RAID_LEADER" or channel == "BATTLEGROUND" or channel == "BATTLEGROUND_LEADER" then
			channel = "RAID"
		end
		if channels[channel] then
			for index, _ in pairs(channels[channel]) do
				if index:lower():match("^" .. word:lower()) then
					tinsert(t, index)
				end
			end
		end
		return t
	end
end

local getNameColor
do
	local sq2 = sqrt(2)
	local pi = _G.math.pi
	local cos = _G.math.cos
	local fmod = _G.math.fmod
	local strbyte = _G.strbyte
	local t = {}

	-- http://www.tecgraf.puc-rio.br/~mgattass/color/HSVtoRGB.htm


	local function HSVtoRGB(h, s, v)
	   if ( s == 0 ) then
		  --achromatic=fail
		  t.r = v
		  t.g = v
		  t.b = v
		  if not t.r then t.r = 0 end
		  if not t.g then t.g = 0 end
		  if not t.b then t.b = 0 end
		  return t.r,t.g,t.b
	   end
	   h = h/60
	   local i = floor(h)
	   local i1 = v * (1 - s)
	   local i2 = v * (1 - s * (h - i))
	   local i3 = v * (1 - s * (1 - (h - i)))
	   if i == 0 then
		  --	return v, i3, i1
		  t.r = v
		  t.g = i3
		  t.b = i1
	   elseif i == 1 then
		  --	return i2, v, i1
		  t.r = i2
		  t.g = v
		  t.b = i1
	   elseif i == 2 then
		  --	return i1, v, i3
		  t.r = i1
		  t.g = v
		  t.b = i3
	   elseif i == 3 then
		  --	return i3, i2, v
		  t.r = i3
		  t.g = i2
		  t.b = v
	   elseif i == 4 then
		  --	return i3, i1, v
		  t.r = i3
		  t.g = i1
		  t.b = v
	   elseif i == 5 then
		  --	return v, i1, i2
		  t.r = v
		  t.g = i1
		  t.b = i2
	   else
		  DEFAULT_CHAT_FRAME:AddMessage("Chatter HSVtoRGB failed")
	   end
	   if not t.r then t.r = 0 end
	   if not t.g then t.g = 0 end
	   if not t.b then t.b = 0 end
	   return t.r,t.g,t.b
	end

	function getNameColor(name)
		local seed = 5381 --old seed: 5124
		local h, s, v = 1, 1, 1
		local r, g, b
		for i = 1, #name do
			seed = 33 * seed + strbyte(name, i) --used to use 29 here
		end
		-- h = fmod(seed, 255) / 255
		h = fmod(seed, 360) --changed the HSVtoRGB to acompany this change
		if (h > 220) and (h < 270) then
		   h = h + 60
		end
		t.r, t.g, t.b = HSVtoRGB(h, s, v)

		return t
	end
end

function Module:GetColor(class, isLocal)
	if isLocal then
		local found = false
		for index, value in pairs(LOCALIZED_CLASS_NAMES_FEMALE) do
			if value == class then class = index; found = true; break; end
		end
		if not found then
			for index, value in pairs(LOCALIZED_CLASS_NAMES_MALE) do
				if value == class then class = index; break; end
			end
		end
	end
	local colors = CUSTOM_CLASS_COLORS and CUSTOM_CLASS_COLORS[class] or RAID_CLASS_COLORS[class]
	return ("%02x%02x%02x"):format(colors.r * 255, colors.g * 255, colors.b * 255)
end

function Module:WipeCache()
	wipe(cache)
end

function Module:UpdateSaveData(v)
	if v then
		for k, v in pairs(localNames) do
			db.realm.names[k] = v
		end
	end
end

local function fixLogin(head, id, misc, who, extra, colon)
	local left, right
	if db.bnetBrackets then
		left	= db.leftBracket
		right	= db.rightBracket
	end
	return head .. id .. misc .. left .. who .. right .. extra .. (misc:match("BN_INLINE_TOAST_ALERT")) and "" or colon
end

local function changeBNetName(misc, id, moreMisc, fakeName, tag, colon)
	local _, charName, _, _, _, _, _, english = BNGetToonInfo(id)
	local left, right = "", ""
	if charName ~= "" then
		if storedName then storedName[id] = charName end
		fakeName = db.noRealNames and charName or fakeName
	else
		if db.noRealNames and storedName and storedName[id] then
			fakeName = storedName[id]
			storedName[id] = nil
		end
	end
	
	if moreMisc:match("BN_INLINE_TOAST_ALERT") then
		misc = misc:sub(1, -2)
	end
	
	if not db.bnetBrackets then
		left	= db.leftBracket
		right	= db.rightBracket
	end
	
	if english ~= "" then
		if not fakeName:match("|cff") then
			if db.nameColoring == "CLASS" then
				fakeName = "|cff" .. Module:GetColor(english, true) .. fakeName .. "|r"
			elseif db.nameColoring == "NAME" then
				fakeName = Module:ColorName(fakeName)
			end
		end
	end
	return misc .. id .. moreMisc .. left .. fakeName .. right .. tag .. ":"
end

local function changeName(header, name, extra, count, display, body)
	if name == player then
		if emphasizeSelfInText then
			body = body:gsub("(" .. player .. ")", "|cffffff00>|r%1|cffffff00<|r"):gsub("(" .. player:lower() .. ")", "|cffffff>|r%1|cffffff00<|r")
		end
		if colorSelfInText then
			body = body:gsub("(" .. player .. ")", "|cffff0000%1|r"):gsub("(" .. player:lower() .. ")", "|cffff0000%1|r")
		end	
	end
	
	if not display:match("|cff") then
		display = Module:ColorName(name)
	end
	
	cache[name] = display
	
	local level
	local tab = db.realm.names[name] or localNames[name]
	if tab then
		level = db.includeLevel and tab.level or nil
	end
	
	if level and (level ~= MAX_PLAYER_LEVEL or not db.excludeMaxLevel) then
		if db.levelByDiff then
			local color = GetQuestDifficultyColor(level)
			level = ("|cff%02x%02x%02x%s|r"):format(color.r * 255, color.g * 255, color.b * 255, level)
		else
			if display:match("|cff......") then
				level = display:gsub(display, "((|cff......).-|r)", function(string, color)
					return ("%s%s|r"):format(color, level)
				end)
			end
		end
		display = ("%s%s%s"):format(display, db.separator, level)
	end
	
	return ("|Hplayer:%s%s%s|h%s%s%s|h%s"):format(name, extra, count, db.leftBracket, display, db.rightBracket, body)
end

function Module:ColorName(name)
	local class
	local tab = db.realm.names[name] or localNames[name]
	if tab then class = tab.class end
	
	if cache[name] then
		name = cache[name]
	else
		if db.nameColoring ~= "NONE" then
			local color = db.nickColor
			if db.nameColoring == "CLASS" then
				color = CUSTOM_CLASS_COLORS and CUSTOM_CLASS_COLORS[class] or RAID_CLASS_COLORS[class] or db.nickColor
			elseif coloring == "NAME" then
				color = getNameColor(name)
			end
			name = ("|cff%02x%02x%02x%s|r"):format(color.r * 255, color.g * 255, color.b * 255, name)
		end
	end
	return name
end

function Module:AddMessage(frame, text, ...)
	if text and type(text) == "string" then
		text = text:gsub("(|Hplayer:([^|:]+)([:%d+]*)([^|]*)|h%[([^%]]+)%]|h)(.-)$", changeName)
		text = text:gsub("(|HBNplayer:%S-|k:)(%d-)(:%S-|h)%[(%S-)%](|?h?)(:?)", changeBNetName)
		text = text:gsub("(|HBNplayer%S-|k)(%d-)(:%S-BN_INLINE_TOAST_ALERT%S-|h)%[(%S-)%](|?h?)(:?)", fixLogin)
	end
	return self.hooks[frame].AddMessage(frame, text, ...)
end

function Module:FRIENDLIST_UPDATE(event)
	for i = 1, GetNumFriends() do
		local name, level, class = GetFriendInfo(i)
		if class then
			self:AddPlayer(name, localClass[class], level, db.saveFriends)
		end
	end
end

function Module:GUILD_ROSTER_UPDATE(event)
	if not IsInGuild() then return end
	wipe(channels.GUILD)
	for i = 1, GetNumGuildMembers() do
		local name, _, _, level, _, _, _, _, online, _, class = GetGuildRosterInfo(i)
		if online then
			channels.GUILD[name] = name
		end
		self:AddPlayer(name, class, level, db.saveGuild)
	end
end

function Module:RAID_ROSTER_UPDATE(event)
	wipe(channels.RAID)
	for i = 1, GetNumRaidMembers() do
		local name, _, _, level, _, class = GetRaidRosterInfo(i)
		if name and class and level then
			channels.RAID[name] = true
			self:AddPlayer(name, class, level, db.saveParty)
		end
	end
end

function Module:PARTY_MEMBERS_CHANGED(event)
	local partyStr = "party%d"
	wipe(channels.PARTY)
	for i = 1, GetNumPartyMembers() do
		local name, level = UnitName(format(playerStr, i)), UnitLevel(format(playerStr, i))
		local _, class = UnitClass(format(playerStr, i))
		channels.PARTY[name] = true
		self:AddPlayer(name, class, level, db.saveParty)
	end
end

function Module:PLAYER_TARGET_CHANGED(event)
	if not UnitExists("target") or not UnitIsPlayer("target") or not UnitIsFriend("player", "target") then return end
	local _, class = UnitClass("target")
	local name, level = UnitName("target"), UnitLevel("target")
	self:AddPlayer(name, class, level, db.saveTarget)
end

function Module:UPDATE_MOUSEOVER_UNIT(event)
	if not UnitExists("mouseover") or not UnitIsPlayer("mouseover") or not UnitIsFriend("player", "mouseover") then return end
	local _, class = UnitClass("mouseover")
	local name, level = UnitName("mouseover"), UnitLevel("mouseover")
	self:AddPlayer(name, class, level, db.saveTarget)
end

function Module:WHO_LIST_UPDATE(event)
	if GetNumWhoResults() <= 3 or db.saveAllWho then
		for i =1, GetNumWhoResults() do
			local name, _, level, _, _, _, class = GetWhoInfo(i)
			if class then
				self:AddPlayer(name, class, level, db.saveWho)
			end
		end
	end
end

function Module:CHAT_MSG_CHANNEL_JOIN(event, _, name, _, _, _, _, _, _, channel)
	channels[channel:lower()] = channels[channel:lower()] or {}
	channels[channel:lower()][name] = true
end

function Module:CHAT_MSG_CHANNEL_LEAVE(event, _, name, _, _, _, _, _, _, channel)
	if not channels[channel:lower()] then return end
	channels[channel:lower()][name] = nil
end
	
function Module:AddPlayer(name, class, level, save)
	if name and class and class ~= UNKNOWN then
		if save or db.realm.names[name] then
			db.realm.names[name] = db.realm.names[name] or {}
			db.realm.names[name].class = class
			if level and level ~= 0 then
				db.realm.names[name].level = level
			end
		else
			localNames[name] = localNames[name] or {}
			localNames[name].class = class
			if level and level ~= 0 then
				localNames[name].level = level
			end
		end
		cache[name] = nil
	end
end

function Module:OnEnable()
	self:RegisterEvent("RAID_ROSTER_UPDATE")
	self:RegisterEvent("PARTY_MEMBERS_CHANGED")
	self:RegisterEvent("WHO_LIST_UPDATE")
	self:RegisterEvent("PLAYER_TARGET_CHANGED")
	self:RegisterEvent("UPDATE_MOUSEOVER_UNIT")
	self:RegisterEvent("CHAT_MSG_SYSTEM", "WHO_LIST_UPDATE")
	self:RegisterEvent("FRIENDLIST_UPDATE")
	self:RegisterEvent("GUILD_ROSTER_UPDATE")
	self:RegisterEvent("CHAT_MSG_CHANNEL_JOIN")
	self:RegisterEvent("CHAT_MSG_CHANNEL_LEAVE")
	self:RegisterEvent("CHAT_MSG_CHANNEL", "CHAT_MSG_CHANNEL_JOIN")
	
	if IsInGuild then GuildRoster() end
	self:RAID_ROSTER_UPDATE()
	self:PARTY_MEMBERS_CHANGED()
	
	for i = 1, NUM_CHAT_WINDOWS do
		local frame = _G[format("ChatFrame%d", i)]
		if frame ~= COMBATLOG then self:RawHook(frame, "AddMessage", true) end
	end
	
	if db.useTabComplete then
		AceTab:RegisterTabCompletion("ElvUI_ChatTweaks", nil, tabComplete)
	end
	
	if CUSTOM_CLASS_COLORS and CUSTOM_CLASS_COLORS.RegisterCallback then
		CUSTOM_CLASS_COLORS:RegisterCallback(wipeCache)
	end
	
	if db.noRealNames then
		local _, n = BNGetNumFriends()
		for i = 1, n do
			local _, _, _, toon, id = BNGetFriendInfo(i)
			storedName[id] = toon
		end
	end
end

function Module:OnDisable()
	if AceTab:IsTabCompletionRegistered("ElvUI_ChatTweaks") then
		AceTab:UnregisterTabCompletion("ElvUI_ChatTweaks")
	end
	
	if CUSTOM_CLASS_COLORS and CUSTOM_CLASS_COLORS.UnregisterCallback then
		CUSTOM_CLASS_COLORS:UnregisterCallback(wipeCache)
	end
end

function Module:OnInitialize()
	self.db = ElvUI_ChatTweaks.db:RegisterNamespace("PlayerNames", defaults)
	for index, value in pairs(self.db.realm.names) do
		if type(value) == "string" then
			self.db.realm.names[index] = {class = v}
		end
	end
	db = self.db.profile
	db.realm = self.db.realm
	db.global = self.db.global
	
	for index, value in pairs(db.global.classes) do
		localClass[value] = index
	end
end

function Module:Info()
	return L["Provides options to color player names, add player levels, and add tab completion of player names."]
end

function Module:GetOptions()
	return options
end